/*
 * AppController.h
 * Classes
 * 
 * Created by Gregory Barchard on 23/09/2010.
 * 
 * Copyright (c) 2010 BuySellAds.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

//
//  BSAAdContainerView.m
//  BSAAdFramework
//
//  Created by Rick Fillion on 6/28/10.
//  Copyright 2010 BuySellAds. All rights reserved.
//

#import "BSAAdContainerView.h"
#import "BSAAdController.h"
#import "NSNotificationCenter+BuySellAds.h"
#import "BSAAdView.h"
#import "BSAAdImageView.h"
#import "BSAAdImageAndTextView.h"
#import "BSAAd.h"
#import "BSASwitchboard.h"
#import "BSAZone.h"
#import "BSAAdViewDelegate.h"
#import "NSMutableArray+Additions.h"

@interface BSAAdContainerView (Private)

- (void)refreshAds;
- (void)frameChanged;
- (void)reflowAdViews;
- (NSUInteger)maximumNumberOfAds;

@end


@implementation BSAAdContainerView

@synthesize zoneIdentifier, backgroundColor, bordered, ads, delegate, hidePlaceholderAds;

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
        self.backgroundColor = [UIColor colorWithWhite:0.1 alpha:0.1];
        self.bordered = YES;
    }
    return self;
}

- (void)awakeFromNib {
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(adsAvailableForZoneIdNotification:) name:BSAAdsAvailableForZoneIdNotification object:nil];
	
	// The current device
	UIDevice *device = [UIDevice currentDevice];
	
	// Generate device orientation notifications
	[device beginGeneratingDeviceOrientationNotifications];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(boundsDidChangeNotification:)
												 name:@"UIDeviceOrientationDidChangeNotification"
											   object:self];
    
    self.hidePlaceholderAds = NO;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:BSAAdsAvailableForZoneIdNotification object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"UIDeviceOrientationDidChangeNotification" object:nil];
	
	// The current device
	UIDevice *device = [UIDevice currentDevice];
	
	// Stop orientation notifications
	[device endGeneratingDeviceOrientationNotifications];
	 
	[ads release];
    [super dealloc];
}

- (BOOL)isFlipped
{
    return YES;
}

- (void)boundsDidChangeNotification:(NSNotification *)notification
{
    [self frameChanged];
}

- (void)frameDidChangeNotification:(NSNotification *)notification
{
    [self frameChanged];
}

- (void)adsAvailableForZoneIdNotification:(NSNotification *)notification
{
    //NSLog(@"adsAvailableForZoneIdNotification: %@", notification);
    NSDictionary *userInfo = [notification userInfo];
    NSNumber *zoneIdentifierForNewAds = [userInfo objectForKey:@"zoneIdentifier"];
    if (!zoneIdentifierForNewAds || !self.zoneIdentifier)
    {
        return;
    }
    if ([zoneIdentifierForNewAds isEqualToNumber:self.zoneIdentifier])
    {
        //NSLog(@"found new ads for me! (%@)", self.zoneIdentifier);
        //NSArray *adsAvailable = [[BSAAdController sharedController] allAdsForZoneId:self.zoneIdentifier];
        //NSLog(@"ads = %@", adsAvailable);
        [self refreshAds];
    }
}

- (void)setZoneIdentifier:(NSNumber *)aZoneIdentifier
{
    [aZoneIdentifier retain];
    [zoneIdentifier release];
    zoneIdentifier = aZoneIdentifier;
    [self refresh];
}

- (void)drawRect:(CGRect)dirtyRect {
    // Drawing code here.
    
    //[[UIColor blueColor] set];
    //NSRectFill([self bounds]);
    
    UIColor *borderColor = [UIColor colorWithRed:0.665 green:0.665 blue:0.665 alpha:1.000];
    UIColor *bgColor = self.backgroundColor ? self.backgroundColor : [UIColor clearColor];
    
    CGFloat stroke = 1.0;
    CGRect bounds = [self bounds];
    bounds.size.width -= 1.0;
    bounds.origin.y += 0.5;
    bounds.size.height -= 1.0; 
    bounds.origin.x += 0.5;
    //bounds = [self pixelAlignRect:bounds withStroke:stroke];
    UIBezierPath *path = [UIBezierPath bezierPathWithRect:bounds];
    [path setLineWidth:stroke];
    [bgColor set];
    [path fill];
    [borderColor set];
    if ([self isBordered])
        [path stroke];
}

- (void)refresh
{
    [self refreshAds];
}

- (Class)adViewClass
{
    BSAZoneFormat zoneFormat = [[BSAAdController sharedController] zoneFormatForZoneId:self.zoneIdentifier];
    
    if (zoneFormat == BSAZoneImageFormat)
        return [BSAAdImageView class];
    else if (zoneFormat == BSAZoneImageAndTextFormat)
        return [BSAAdImageAndTextView class];
    else {
        NSLog(@"[BSAAdController adViewClass] encountered a zoneFormat not supported: %i", zoneFormat);
        return nil;
    }
    
}

#pragma mark Private


- (void)refreshAds
{
    [self willChangeValueForKey:@"ads"];
    [ads release];
    NSNumber *howMany = [[BSAAdController sharedController] numberOfAdsForZoneId:self.zoneIdentifier];
    ads = [[BSAAdController sharedController] randomAdsForZoneId:self.zoneIdentifier howMany:[howMany integerValue]];
    [ads retain];    
    [self didChangeValueForKey:@"ads"];
    [self reflowAdViews];
    
    // Register the impressions.
    NSMutableArray *adIdentifiers = [NSMutableArray array];
    for (BSAAd *ad in ads)
    {
        if (ad.identifier)
            [adIdentifiers addObject:ad.identifier];
    }
    [[BSASwitchboard switchboard] registerImpressionForAdIds:adIdentifiers zoneId:self.zoneIdentifier target:self selector:@selector(impressionResponse:error:)];
}

- (void)frameChanged
{
    if ([ads count] < [self maximumNumberOfAds])
    {
        [self refreshAds];
    }
    else {
        [self reflowAdViews];
    }
}

- (void)reflowAdViews
{
    //NSLog(@"reflowAdViews");
    
    // get rid of all the old adViews;
    [[self subviews] makeObjectsPerformSelector:@selector(removeFromSuperview)];
    
    CGSize adSize = [[BSAAdController sharedController] sizeForZoneId:self.zoneIdentifier];
    if (CGSizeEqualToSize(adSize, CGSizeZero))
    {
        return;
    }
    
    BSAZoneFormat zoneFormat = [[BSAAdController sharedController] zoneFormatForZoneId:self.zoneIdentifier];
    
    // figure out how many ads we can fit horizontally
    NSUInteger boundsWidth = (NSUInteger)[self bounds].size.width;
    NSUInteger boundsHeight = (NSUInteger)[self bounds].size.height - ([self isBordered] ? 2.0 : 0.0);
    NSUInteger adWidth = (NSUInteger)adSize.width;
    if (zoneFormat == BSAZoneImageAndTextFormat)
    {
        adWidth = boundsWidth - 2.0;
    }
    NSUInteger adHeight = (NSUInteger)adSize.height;
    
    NSUInteger maximumNumberOfAds = [self maximumNumberOfAds];
    
    // Hide from view if no ads are to be display and we're set to hide placeholder ads
    BOOL hasBackfill = [[BSAAdController sharedController] hasBackfillAdsForZoneId:self.zoneIdentifier];
    self.hidden = self.hidePlaceholderAds && (maximumNumberOfAds == 0) && !hasBackfill;
    
    NSUInteger horizontalAmount = boundsWidth / adWidth;
    if (maximumNumberOfAds < horizontalAmount)
        horizontalAmount = maximumNumberOfAds;
    if (horizontalAmount == 0)
        return; // lets avoid some divide by zero action.
    NSUInteger verticalAmount =  [self maximumNumberOfAds] / horizontalAmount;
    if (maximumNumberOfAds > 0 && verticalAmount == 0)
        verticalAmount = 1;
    
    //NSLog(@"horizontalAmount = %i, verticalAmount = %i", horizontalAmount, verticalAmount);
    
    // figure out spacing between ads
    CGFloat horizontalSpacing = ((CGFloat)(boundsWidth - (adWidth * horizontalAmount))) / ((CGFloat) horizontalAmount+1);
    CGFloat verticalSpacing = ((CGFloat)(boundsHeight - (adHeight * verticalAmount))) / ((CGFloat) verticalAmount+1);
    
    CGPoint origin = CGPointZero;
    origin.y = ([self isBordered] ? 1.0 : 0.0) + verticalSpacing;
    NSInteger adIndex = 0;
    Class adViewClass = [self adViewClass];
    for (int v = 0; v < verticalAmount; v++)
    {
        origin.x = horizontalSpacing;
        for (int h = 0; h < horizontalAmount; h++)
        {
            if (adIndex < [ads count])
            {
                BSAAd *ad = [ads objectAtIndex:adIndex];
                CGRect frameRect = CGRectMake(origin.x, origin.y, adWidth, adHeight);
                //frameRect = [self pixelAlignRect:frameRect withStroke:0];
                //NSLog(@"creating new BSAAdView for zone id %@ with frame %@", self.zoneIdentifier, NSStringFromRect(frameRect));
                
                // Backfill the container
                if ([ad isPlaceholder] && hasBackfill)
                {
                    // 
                    NSDictionary *backfill = [[BSAAdController sharedController] backfillAdsForZoneId:self.zoneIdentifier];
                    NSUInteger count = [backfill count];
                    NSArray *shuffledBackfill = [backfill keysSortedByValueUsingComparator:
                                                 ^NSComparisonResult(id obj1, id obj2)
                    {
                        CGFloat w1 = [obj1 floatValue]*(random()%count);
                        CGFloat w2 = [obj2 floatValue]*(random()%count);
                        if (w1 < w2)
                            return NSOrderedAscending;
                        else if (w1 > w2)
                            return NSOrderedDescending;
                        else
                            return NSOrderedSame;   
                    }];
                    
                    //
                    UIWebView *webView = [[[UIWebView alloc] initWithFrame:frameRect] autorelease];
                    [webView loadRequest:[NSURLRequest requestWithURL:[shuffledBackfill objectAtIndex:0]]];
                    
                    //
                    [webView setDelegate:(id <UIWebViewDelegate>)self];
                    [self addSubview:webView];
                }
                else {
                    //
                    BSAAdView *adView =  [[(BSAAdView *)[adViewClass alloc] initWithFrame:frameRect] autorelease];
                    adView.ad = ad;
                    adView.delegate = self;
                    [self addSubview:adView];
                }
                
                origin.x += adSize.width + horizontalSpacing;
            }
            adIndex++;
        }
        origin.y += adSize.height + verticalSpacing;
    }
}

- (NSUInteger)maximumNumberOfAds
{
    CGSize adSize = [[BSAAdController sharedController] sizeForZoneId:self.zoneIdentifier];
    if (CGSizeEqualToSize(adSize, CGSizeZero))
    {
        return 0;
    }
    
    BSAZoneFormat zoneFormat = [[BSAAdController sharedController] zoneFormatForZoneId:self.zoneIdentifier];
    
    // figure out how many ads we can fit horizontally
    NSUInteger boundsWidth = (NSUInteger)[self bounds].size.width;
    NSUInteger boundsHeight = (NSUInteger)[self bounds].size.height;
    NSUInteger adWidth = (NSUInteger)adSize.width;
    if (zoneFormat == BSAZoneImageAndTextFormat)
    {
        adWidth = boundsWidth - 2.0;
    }
    NSUInteger adHeight = (NSUInteger)adSize.height;
    
    NSUInteger horizontalAmount = boundsWidth / adWidth;
    
    NSUInteger verticalAmount = boundsHeight / adHeight;
    NSUInteger maxAmountThatCanFit = horizontalAmount * verticalAmount;
    //NSLog(@"%i %i %i %i", horizontalAmount, verticalAmount, adWidth, adHeight);
    
    NSInteger proposedNumberOfAdsForZone = [[[BSAAdController sharedController] numberOfAdsForZoneId:self.zoneIdentifier] integerValue];
    NSInteger actualNumberofAdsAvailableForZone = [[[BSAAdController sharedController] allAdsForZoneId:self.zoneIdentifier] count];
    BOOL showsPlaceholderAds = [[BSAAdController sharedController] showsPlaceholderAdForZoneId:self.zoneIdentifier];
    BOOL repeatsPlaceholderAds = [[BSAAdController sharedController] repeatsPlaceholderAdsForZoneId:self.zoneIdentifier];
    
    NSUInteger maximumNumberOfAds = 0;
    
    if (proposedNumberOfAdsForZone <= actualNumberofAdsAvailableForZone)
    {
        // We have more ads than what the advertiser wants.  So now we just need to 
        // Figure out if we can fit that many in the view
        if (maxAmountThatCanFit < proposedNumberOfAdsForZone)
            maximumNumberOfAds = maxAmountThatCanFit;
        else
            maximumNumberOfAds = proposedNumberOfAdsForZone;
    }
    else {
        // There aren't enough ads.
        if (!showsPlaceholderAds)
        {
            // Doesn't want placeholders so just use what we have.
            maximumNumberOfAds = actualNumberofAdsAvailableForZone;
        }
        else {
            // Does want placeholders
            if (!repeatsPlaceholderAds)
            {
                // Only wants one placeholder
                maximumNumberOfAds =  actualNumberofAdsAvailableForZone + 1;
            }
            else {
                // Wants as many placeholders as will fit, up to the max proposed
                if (maxAmountThatCanFit < proposedNumberOfAdsForZone)
                    maximumNumberOfAds = maxAmountThatCanFit;
                else 
                    maximumNumberOfAds = proposedNumberOfAdsForZone;
            }
        }
    }
    return maximumNumberOfAds;
}


#pragma mark BSAAdViewDelegate

- (void)adClicked:(BSAAd *)ad;
{
    //NSLog(@"ad clicked:  %@", ad);
    if (ad.identifier)
    {
        // It's a real ad.
        BOOL shouldHandle = YES;
        if (self.delegate && [(id)self.delegate respondsToSelector:@selector(adContainerView:shouldHandleClickOfAd:)])
        {
            shouldHandle = [self.delegate adContainerView:self shouldHandleClickOfAd:ad];
        }
        
        if (shouldHandle)
            [[UIApplication sharedApplication] openURL:[NSURL URLWithString:ad.url]];
        
        // Report the click
        [[BSASwitchboard switchboard] registerClickForAdId:ad.identifier zoneId:self.zoneIdentifier target:self selector:@selector(clickResponse:error:)];
    }
    else {
        // It's a spot that's for sale.
        NSString *urlString = [[BSAAdController sharedController] buyUrlForZoneId:self.zoneIdentifier];
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:urlString]];
    }
}

#pragma mark -
#pragma mark WebView delegate
-(BOOL) webView:(UIWebView *)inWeb shouldStartLoadWithRequest:(NSURLRequest *)inRequest navigationType:(UIWebViewNavigationType)inType {
    
    // Open links in Safari
    // ref: http://stackoverflow.com/questions/2899699/uiwebview-open-links-in-safari
    if ( inType == UIWebViewNavigationTypeLinkClicked ) {
        [[UIApplication sharedApplication] openURL:[inRequest URL]];
        return NO;
    }
    
    return YES;
}

#pragma mark BSASwitchboard Responses

// Click and Impression responses return gif data which throws JSON for a loop.
// It'll always return an error.

- (void)clickResponse:(id)response error:(NSError *)error
{

}

- (void)impressionResponse:(id)response error:(NSError *)error
{

}

@end
