/*
 * AppController.h
 * Classes
 * 
 * Created by Gregory Barchard on 23/09/2010.
 * 
 * Copyright (c) 2010 BuySellAds.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

//
//  BSAAdImageAndTextView.m
//  BSAAdFramework
//
//  Created by Rick Fillion on 7/2/10.
//  Copyright 2010 BuySellAds. All rights reserved.
//

#import "BSAAdImageAndTextView.h"
#import "BSAAdView+Private.h"
#import "BSAAdImageView+Private.h"
#import "BSAMiddleAlignedTextField.h"
#import "BSAAd.h"


@implementation BSAAdImageAndTextView

@synthesize titleTextField, descriptionTextField;

- (void)customInit
{
    [super customInit];
    descriptionTextField = [[BSAMiddleAlignedTextField alloc] initWithFrame:CGRectZero];
	[descriptionTextField setFont:[UIFont systemFontOfSize:11.0]];
	descriptionTextField.editable = NO;
    [self addSubview:descriptionTextField];
	
    titleTextField = [[BSAMiddleAlignedTextField alloc] initWithFrame:CGRectZero];
    [titleTextField setFont:[UIFont boldSystemFontOfSize:12.0]];
	titleTextField.editable = NO;
    [self addSubview: titleTextField];
	
	self.contentHorizontalAlignment = UITextAlignmentLeft;
	
	// Move the image view to the left	
	CGFloat offset = [self contentRectForBounds:[self bounds]].size.width/2.0; //[self bounds].size.width/2.0;
	UIEdgeInsets inset = UIEdgeInsetsMake(0, -offset, 0, 0);
	UIEdgeInsets outset = UIEdgeInsetsMake(0, offset, 0, 0);
	[self setImageEdgeInsets:inset];
	[self setTitleEdgeInsets:outset];
	
    textPadding = 3.0;
}

- (void)dealloc
{
    [descriptionTextField removeFromSuperview];
    [descriptionTextField release];
    [titleTextField removeFromSuperview];
    [titleTextField release];
    [super dealloc];    
}

- (void)setAd:(BSAAd *)newAd
{
    [super setAd:newAd];
    titleTextField.text = (ad.title ? ad.title : @"");
    descriptionTextField.text = (ad.text ? ad.text : @"");

    [self updateTextFieldFrames];
}

- (void)updateTextFieldFrames
{	
	[titleTextField sizeToFit];
	[descriptionTextField sizeToFit];
	
	// Get the size of the ad
	CGSize adSize = self.ad.image.size;
	
	// Get the bounds for drawing
	CGRect drawingRect = [self contentRectForBounds:[self bounds]];
	
	// The offset for the text fields
	
	// The X offset is the same
	CGFloat xOffset = adSize.width + 3*textPadding;
	
	// The Y offset is different for each
	CGFloat yOffsetTitle = 0;
	CGFloat yOffsetDescription = titleTextField.contentSize.height + textPadding;
	
	// The width
	
	// The widths are the same
	CGFloat width = drawingRect.size.width - adSize.width - 3*textPadding;
	
	// The heights are different
	CGFloat heightTitle = titleTextField.contentSize.height;
	CGFloat heightDescription = drawingRect.size.height - heightTitle - textPadding;
	[descriptionTextField setFrame:CGRectMake(xOffset, yOffsetDescription, width, heightDescription)];
	[titleTextField setFrame:CGRectMake(xOffset, yOffsetTitle, width, heightTitle)];
}

#pragma mark BSAAdImageView+Private Overrides

- (void)updateImage
{
    [super updateImage];
    [self updateTextFieldFrames];
}

#ifdef CONFIGURATION_DEBUG
// Debug code to draw an outline around the view
- (void)drawRect:(CGRect)rect {
	[super drawRect:rect];
	UIBezierPath *path = [UIBezierPath bezierPathWithRect:[self bounds]];
	[[UIColor blackColor] set];
	[path setLineWidth:1.0];
	[path stroke];
}
#endif

@end
