/*
 * AppController.h
 * Classes
 * 
 * Created by Gregory Barchard on 23/09/2010.
 * 
 * Copyright (c) 2010 BuySellAds.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

//
//  BSAAdView+Private.m
//  BSAAdFramework
//
//  Created by Rick Fillion on 7/2/10.
//  Copyright 2010 BuySellAds. All rights reserved.
//

#import "BSAAdView+Private.h"
#import "BSAAd.h"
//#import "LTPixelAlign.h"

@implementation BSAAdView (Private)

- (void)customInit
{
	[self addTarget:self
			 action:@selector(adClicked:)
   forControlEvents:UIControlEventTouchUpInside];
	[self setTitle:@""
		  forState:UIControlStateNormal];
	[self setImage:[self defaultImage]
		  forState:UIControlStateNormal];
}

- (void)adClicked:(id)sender
{
    if (delegate && ![(id)delegate respondsToSelector:@selector(adClicked:)])
    {
        NSLog(@"delegate set that doesn't respond to adClicked:");
        return;
    }
    [delegate adClicked:self.ad];
}

- (UIImage *)defaultImage
{	
	// Get the graphics context ready for drawing
	UIGraphicsBeginImageContext([self bounds].size);
	
    {
        if (self.ad.identifier) {
            [self drawDefaultImageWithString:@"Loading..."];
        } else {
            [self drawDefaultImageWithString:@"Advertise Here"];
        }
    }
	
	// Get an autoreleased image from the current graphics context
	UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
	
	// Clean up the context
	UIGraphicsEndImageContext();
	
	// Return the image
    return image;
}

- (void)drawDefaultImageWithString:(NSString *)string
{
	// The bounds to draw in
	CGRect bounds = [self bounds];
	
	// The stroke size
	CGFloat stroke = 1.0;
	
	// Adjust the bounds to account for the stroke
	bounds.size.width -= 2*stroke;
	bounds.size.height -= 2*stroke;
	bounds.origin.x += stroke;
	bounds.origin.y += stroke;
	
	// The colors for drawing
	UIColor *borderColor = [UIColor colorWithRed:0.756
										   green:0.756
											blue:0.755
										   alpha:1.000];
	UIColor *bgColor = [UIColor colorWithRed:0.883
									   green:0.883
										blue:0.883
									   alpha:1.000];
	UIColor *textColor = [UIColor colorWithRed:0.327
										 green:0.327
										  blue:0.327
										 alpha:1.000];
	
	// The bezierpath for drawing
	UIBezierPath *path = [UIBezierPath bezierPathWithRect:bounds];
	[path setLineWidth:stroke];
	
	[bgColor set];
	[path fill];
	
	[borderColor set];
	[path stroke];
	
	// Add a label
	UILabel *label = [[[UILabel alloc] initWithFrame:bounds] autorelease];
	label.text = string;
	label.textColor = textColor;
	label.textAlignment = UITextAlignmentCenter;
    
    // The font for the label
	//UIFont *font = [UIFont fontWithName:@"Helvetica Neue" size:10.0];
    //label.font = font;
    
	[label drawTextInRect:bounds];
}



@end
